package com.sun.org.apache.xml.internal.security.c14n.implementations;

import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import com.sun.org.apache.xml.internal.security.utils.XMLUtils;
import java.io.IOException;
import org.w3c.dom.DOMException;
import java.io.OutputStream;
import java.util.Iterator;
import org.w3c.dom.NamedNodeMap;
import com.sun.org.apache.xml.internal.security.c14n.helper.C14nHelper;
import org.w3c.dom.Attr;
import java.util.TreeSet;
import java.util.Map;
import org.w3c.dom.Element;
import java.util.Set;
import com.sun.org.apache.xml.internal.security.signature.XMLSignatureInput;
import com.sun.org.apache.xml.internal.security.transforms.params.InclusiveNamespaces;
import com.sun.org.apache.xml.internal.security.c14n.CanonicalizationException;
import org.w3c.dom.Node;
import java.util.SortedSet;

public abstract class Canonicalizer20010315Excl extends CanonicalizerBase
{
    private SortedSet<String> inclusiveNSSet;
    private boolean propagateDefaultNamespace;
    
    public Canonicalizer20010315Excl(boolean b) {
        super(b);
        this.propagateDefaultNamespace = false;
    }
    
    @Override
    public byte[] engineCanonicalizeSubTree(Node node) throws CanonicalizationException {
        return this.engineCanonicalizeSubTree(node, "", null);
    }
    
    public byte[] engineCanonicalizeSubTree(Node node, String s) throws CanonicalizationException {
        return this.engineCanonicalizeSubTree(node, s, null);
    }
    
    public byte[] engineCanonicalizeSubTree(Node node, String s, boolean propagateDefaultNamespace) throws CanonicalizationException {
        this.propagateDefaultNamespace = propagateDefaultNamespace;
        return this.engineCanonicalizeSubTree(node, s, null);
    }
    
    public byte[] engineCanonicalizeSubTree(Node node, String inclusiveNamespaces, Node node2) throws CanonicalizationException {
        this.inclusiveNSSet = InclusiveNamespaces.prefixStr2Set(inclusiveNamespaces);
        return super.engineCanonicalizeSubTree(node, node2);
    }
    
    public byte[] engineCanonicalize(XMLSignatureInput xmlSignatureInput, String inclusiveNamespaces) throws CanonicalizationException {
        this.inclusiveNSSet = InclusiveNamespaces.prefixStr2Set(inclusiveNamespaces);
        return super.engineCanonicalize(xmlSignatureInput);
    }
    
    public byte[] engineCanonicalizeXPathNodeSet(Set<Node> set, String inclusiveNamespaces) throws CanonicalizationException {
        this.inclusiveNSSet = InclusiveNamespaces.prefixStr2Set(inclusiveNamespaces);
        return super.engineCanonicalizeXPathNodeSet(set);
    }
    
    protected void outputAttributesSubtree(Element element, NameSpaceSymbTable nameSpaceSymbTable, Map<String, byte[]> map) throws CanonicalizationException, DOMException, IOException {
        TreeSet set = new TreeSet(Canonicalizer20010315Excl.COMPARE);
        TreeSet<String> set2 = new TreeSet();
        if (this.inclusiveNSSet != null && !this.inclusiveNSSet.isEmpty()) {
            set2.addAll(this.inclusiveNSSet);
        }
        if (element.hasAttributes()) {
            NamedNodeMap attributes = element.getAttributes();
            for (int length = attributes.getLength(), i = 0; i < length; ++i) {
                Attr attr = (Attr)attributes.item(i);
                String localName = attr.getLocalName();
                String nodeValue = attr.getNodeValue();
                if (!"http://www.w3.org/2000/xmlns/".equals(attr.getNamespaceURI())) {
                    String prefix = attr.getPrefix();
                    if (prefix != null && !prefix.equals("xml") && !prefix.equals("xmlns")) {
                        set2.add(prefix);
                    }
                    set.add(attr);
                }
                else if ((!"xml".equals(localName) || !"http://www.w3.org/XML/1998/namespace".equals(nodeValue)) && nameSpaceSymbTable.addMapping(localName, nodeValue, attr) && C14nHelper.namespaceIsRelative(nodeValue)) {
                    throw new CanonicalizationException("c14n.Canonicalizer.RelativeNamespace", new Object[] { element.getTagName(), localName, attr.getNodeValue() });
                }
            }
        }
        if (this.propagateDefaultNamespace && nameSpaceSymbTable.getLevel() == 1 && this.inclusiveNSSet.contains("xmlns") && nameSpaceSymbTable.getMappingWithoutRendered("xmlns") == null) {
            nameSpaceSymbTable.removeMapping("xmlns");
            nameSpaceSymbTable.addMapping("xmlns", "", this.getNullNode(element.getOwnerDocument()));
        }
        String prefix2;
        if (element.getNamespaceURI() != null && element.getPrefix() != null && element.getPrefix().length() != 0) {
            prefix2 = element.getPrefix();
        }
        else {
            prefix2 = "xmlns";
        }
        set2.add(prefix2);
        Iterator<Object> iterator = set2.iterator();
        while (iterator.hasNext()) {
            Attr mapping = nameSpaceSymbTable.getMapping((String)iterator.next());
            if (mapping != null) {
                set.add(mapping);
            }
            else {
                continue;
            }
        }
        OutputStream writer = this.getWriter();
        for (Attr attr2 : set) {
            CanonicalizerBase.outputAttrToWriter(attr2.getNodeName(), attr2.getNodeValue(), writer, map);
        }
    }
    
    protected void outputAttributes(Element element, NameSpaceSymbTable nameSpaceSymbTable, Map<String, byte[]> map) throws CanonicalizationException, DOMException, IOException {
        TreeSet set = new TreeSet(Canonicalizer20010315Excl.COMPARE);
        TreeSet<String> set2 = null;
        int n = this.isVisibleDO(element, nameSpaceSymbTable.getLevel()) == 1;
        if (!(n == 0)) {
            set2 = new TreeSet();
            if (this.inclusiveNSSet != null && !this.inclusiveNSSet.isEmpty()) {
                set2.addAll(this.inclusiveNSSet);
            }
        }
        if (element.hasAttributes()) {
            NamedNodeMap attributes = element.getAttributes();
            for (int length = attributes.getLength(), i = 0; i < length; ++i) {
                Attr namespace = (Attr)attributes.item(i);
                String localName = namespace.getLocalName();
                String nodeValue = namespace.getNodeValue();
                if (!"http://www.w3.org/2000/xmlns/".equals(namespace.getNamespaceURI())) {
                    if (this.isVisible(namespace) && n) {
                        String prefix = namespace.getPrefix();
                        if (prefix != null && !prefix.equals("xml") && !prefix.equals("xmlns")) {
                            set2.add(prefix);
                        }
                        set.add(namespace);
                    }
                }
                else if (n && !this.isVisible(namespace) && !"xmlns".equals(localName)) {
                    nameSpaceSymbTable.removeMappingIfNotRender(localName);
                }
                else {
                    if (n == 0 && this.isVisible(namespace) && this.inclusiveNSSet.contains(localName) && !nameSpaceSymbTable.removeMappingIfRender(localName)) {
                        Node addMappingAndRender = nameSpaceSymbTable.addMappingAndRender(localName, nodeValue, namespace);
                        if (addMappingAndRender != null) {
                            set.add((Attr)addMappingAndRender);
                            if (C14nHelper.namespaceIsRelative(namespace)) {
                                throw new CanonicalizationException("c14n.Canonicalizer.RelativeNamespace", new Object[] { element.getTagName(), localName, namespace.getNodeValue() });
                            }
                        }
                    }
                    if (nameSpaceSymbTable.addMapping(localName, nodeValue, namespace) && C14nHelper.namespaceIsRelative(nodeValue)) {
                        throw new CanonicalizationException("c14n.Canonicalizer.RelativeNamespace", new Object[] { element.getTagName(), localName, namespace.getNodeValue() });
                    }
                }
            }
        }
        if (!(n == 0)) {
            Attr attributeNodeNS = element.getAttributeNodeNS("http://www.w3.org/2000/xmlns/", "xmlns");
            if (attributeNodeNS != null && !this.isVisible(attributeNodeNS)) {
                nameSpaceSymbTable.addMapping("xmlns", "", this.getNullNode(attributeNodeNS.getOwnerDocument()));
            }
            String prefix2;
            if (element.getNamespaceURI() != null && element.getPrefix() != null && element.getPrefix().length() != 0) {
                prefix2 = element.getPrefix();
            }
            else {
                prefix2 = "xmlns";
            }
            set2.add(prefix2);
            Iterator<Object> iterator = set2.iterator();
            while (iterator.hasNext()) {
                Attr mapping = nameSpaceSymbTable.getMapping((String)iterator.next());
                if (mapping != null) {
                    set.add(mapping);
                }
                else {
                    continue;
                }
            }
        }
        OutputStream writer = this.getWriter();
        for (Attr attr : set) {
            CanonicalizerBase.outputAttrToWriter(attr.getNodeName(), attr.getNodeValue(), writer, map);
        }
    }
    
    protected void circumventBugIfNeeded(XMLSignatureInput xmlSignatureInput) throws CanonicalizationException, ParserConfigurationException, IOException, SAXException {
        if (!xmlSignatureInput.isNeedsToBeExpanded() || this.inclusiveNSSet.isEmpty()) {
            return;
        }
        Document doc;
        if (xmlSignatureInput.getSubNode() != null) {
            doc = XMLUtils.getOwnerDocument(xmlSignatureInput.getSubNode());
        }
        else {
            doc = XMLUtils.getOwnerDocument(xmlSignatureInput.getNodeSet());
        }
        XMLUtils.circumventBug2650(doc);
        return;
    }
}
